@@ -4,7 +4,7 @@ package JSON::Schema::Modern::Vocabulary::Core;
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: Implementation of the JSON Schema Core vocabulary
 
-our $VERSION = '0.591';
+our $VERSION = '0.592';
 
 use 5.020;
 use Moo;
@@ -31,14 +31,17 @@ sub evaluation_order ($class) { 0 }
 
 sub keywords ($class, $spec_version) {
   return (
-    qw($id $schema),
-    $spec_version ne 'draft7' ? '$anchor' : (),
+    $spec_version eq 'draft4' ? 'id' : '$id',
+    '$schema',
+    $spec_version !~ /^draft[467]$/ ? '$anchor' : (),
     $spec_version eq 'draft2019-09' ? '$recursiveAnchor' : (),
-    $spec_version eq 'draft2020-12' ? '$dynamicAnchor' : (),
+    $spec_version !~ /^draft(?:[467]|2019-09)$/ ? '$dynamicAnchor' : (),
     '$ref',
     $spec_version eq 'draft2019-09' ? '$recursiveRef' : (),
-    $spec_version eq 'draft2020-12' ? '$dynamicRef' : (),
-    $spec_version eq 'draft7' ? 'definitions' : qw($vocabulary $comment $defs),
+    $spec_version !~ /^draft(?:[467]|2019-09)$/ ? '$dynamicRef' : (),
+    $spec_version !~ /^draft[467]$/ ? '$vocabulary' : (),
+    $spec_version =~ /^draft[467]$/ ? 'definitions' : '$defs',
+    $spec_version !~ /^draft[46]$/ ? '$comment' : (),
   );
 }
 
@@ -51,23 +54,23 @@ sub _traverse_keyword_id ($class, $schema, $state) {
   return if not assert_keyword_type($state, $schema, 'string')
     or not assert_uri_reference($state, $schema);
 
-  my $uri = Mojo::URL->new($schema->{'$id'});
+  my $uri = Mojo::URL->new($schema->{$state->{keyword}});
 
-  if ($state->{spec_version} eq 'draft7') {
+  if ($state->{spec_version} =~ /^draft[467]$/) {
     if (length($uri->fragment)) {
-      return E($state, '$id cannot change the base uri at the same time as declaring an anchor')
+      return E($state, '%s cannot change the base uri at the same time as declaring an anchor', $state->{keyword})
         if length($uri->clone->fragment(undef));
 
       return $class->_traverse_keyword_anchor({ %$schema, $state->{keyword} => $uri->fragment }, $state);
     }
   }
   else {
-    return E($state, '$id value "%s" cannot have a non-empty fragment', $schema->{'$id'})
+    return E($state, '%s value "%s" cannot have a non-empty fragment', $state->{keyword}, $schema->{$state->{keyword}})
       if length $uri->fragment;
   }
 
   $uri->fragment(undef);
-  return E($state, '$id cannot be empty') if not length $uri;
+  return E($state, '%s cannot be empty', $state->{keyword}) if not length $uri;
 
   $state->{initial_schema_uri} = $uri->is_abs ? $uri : $uri->to_abs($state->{initial_schema_uri});
   $state->{traversed_schema_path} = $state->{traversed_schema_path}.$state->{schema_path};
@@ -119,7 +122,7 @@ sub _traverse_keyword_anchor ($class, $schema, $state) {
   return if not assert_keyword_type($state, $schema, 'string');
 
   return E($state, '%s value "%s" does not match required syntax',
-      $state->{keyword}, ($state->{keyword} eq '$id' ? '#' : '').$schema->{$state->{keyword}})
+      $state->{keyword}, ($state->{keyword} eq '$anchor' ? '' : '#').$schema->{$state->{keyword}})
     if $state->{spec_version} =~ /^draft(?:7|2019-09)$/
         and $schema->{$state->{keyword}} !~ /^[A-Za-z][A-Za-z0-9_:.-]*$/
       or $state->{spec_version} eq 'draft2020-12'
@@ -287,7 +290,7 @@ JSON::Schema::Modern::Vocabulary::Core - Implementation of the JSON Schema Core
 
 =head1 VERSION
 
-version 0.591
+version 0.592
 
 =head1 DESCRIPTION
 
@@ -311,6 +314,14 @@ the equivalent Draft 2019-09 keywords, indicated in metaschemas with the URI C<h
 
 the equivalent Draft 7 keywords that correspond to this vocabulary and are formally specified in L<https://datatracker.ietf.org/doc/html/draft-handrews-json-schema-01>.
 
+=item *
+
+the equivalent Draft 6 keywords that correspond to this vocabulary and are formally specified in L<https://json-schema.org/draft-06/draft-wright-json-schema-01>.
+
+=item *
+
+the equivalent Draft 4 keywords that correspond to this vocabulary and are formally specified in L<https://json-schema.org/draft-04/draft-zyp-json-schema-04>.
+
 =back
 
 =head1 SUPPORT
